"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockSingleLink = exports.OperationNameMockLink = void 0;
/**
 * A more flexible mockLink for Apollo, that can work based on operationName only
 *
 * Allow to mock based on operation name only
 * Usage: <MockedProvider link={mockLink(mocks, false)}><YourComponent></MockProvider>
 *
 * NOTE: if you have multiple mocks for the same operation, you can still add variables to differentiate them
 * NOTE2: it WON'T work as expected if you need to distinguish mocks for operations with the same name, in case of sucharching operators
 *
 * @see https://github.com/apollographql/apollo-client/blob/main/src/utilities/testing/mocking/mockLink.ts
 */
const graphql_1 = require("graphql");
const equality_1 = require("@wry/equality");
const ts_invariant_1 = require("ts-invariant");
const client_1 = require("@apollo/client"); // '../../../link/core';
// import cloneDeep from "lodash/cloneDeep";
const utilities_1 = require("@apollo/client/utilities");
/** Serialize the query to make it a key */
function requestToKey(request, addTypename) {
    var _a;
    // We use the operationName instead of the full query as key
    return (_a = request.operationName) !== null && _a !== void 0 ? _a : "";
    //const queryString =
    //  request.query &&
    //  print(addTypename ? addTypenameToDocument(request.query) : request.query);
    //const requestKey = { query: queryString };
    //return JSON.stringify(requestKey);
}
class OperationNameMockLink extends client_1.ApolloLink {
    constructor(mockedResponses, addTypename = true) {
        super();
        this.addTypename = true;
        this.mockedResponsesByKey = {};
        this.addTypename = addTypename;
        if (mockedResponses) {
            mockedResponses.forEach((mockedResponse) => {
                this.addMockedResponse(mockedResponse);
            });
        }
    }
    addMockedResponse(mockedResponse) {
        const normalizedMockedResponse = this.normalizeMockedResponse(mockedResponse);
        const key = requestToKey(normalizedMockedResponse.request, this.addTypename);
        let mockedResponses = this.mockedResponsesByKey[key];
        if (!mockedResponses) {
            mockedResponses = [];
            this.mockedResponsesByKey[key] = mockedResponses;
        }
        mockedResponses.push(normalizedMockedResponse);
    }
    request(operation) {
        this.operation = operation;
        const key = requestToKey(operation, this.addTypename);
        let responseIndex = 0;
        const mockedResponses = this.mockedResponsesByKey[key] || [];
        const response = mockedResponses.find((res, index) => {
            const mockedResponseVariables = res.request.variables;
            // If variables are specified in the mock, use them to compare to the query
            if (mockedResponseVariables) {
                const requestVariables = operation.variables || {};
                if (equality_1.equal(requestVariables, mockedResponseVariables)) {
                    responseIndex = index;
                    return true;
                }
                return false;
            }
            // If variable are not specified, return the first matching query
            console.log(`No variables specified for operation ${operation.operationName}, using the first mock found.`);
            responseIndex = index;
            return true;
        });
        let configError;
        if (!response || typeof responseIndex === "undefined") {
            configError = new Error(`No more mocked responses for the query: ${graphql_1.print(operation.query)}, variables: ${JSON.stringify(operation.variables)}. Possible variables were: ${JSON.stringify(mockedResponses.map((r) => { var _a; return (_a = r === null || r === void 0 ? void 0 : r.request) === null || _a === void 0 ? void 0 : _a.variables; }))}`);
        }
        else {
            // Side effet that seems to drop the previously seen mock
            // TODO: should we reenable it? Difficult to tell when Storybook is regenerating the mocks in this case
            this.mockedResponsesByKey[key].splice(responseIndex, 1);
            const { newData } = response;
            if (newData) {
                response.result = newData();
                this.mockedResponsesByKey[key].push(response);
            }
            if (!response.result && !response.error) {
                configError = new Error(`Mocked response should contain either result or error: ${key}`);
            }
        }
        return new client_1.Observable((observer) => {
            const timer = setTimeout(() => {
                if (configError) {
                    try {
                        // The onError function can return false to indicate that
                        // configError need not be passed to observer.error. For
                        // example, the default implementation of onError calls
                        // observer.error(configError) and then returns false to
                        // prevent this extra (harmless) observer.error call.
                        // @ts-ignore
                        if (this.onError(configError, observer) !== false) {
                            throw configError;
                        }
                    }
                    catch (error) {
                        observer.error(error);
                    }
                }
                else if (response) {
                    if (response.error) {
                        observer.error(response.error);
                    }
                    else {
                        if (response.result) {
                            observer.next(typeof response.result === "function"
                                ? response.result()
                                : response.result);
                        }
                        observer.complete();
                    }
                }
            }, (response && response.delay) || 0);
            return () => {
                clearTimeout(timer);
            };
        });
    }
    normalizeMockedResponse(mockedResponse) {
        const newMockedResponse = utilities_1.cloneDeep(mockedResponse);
        // TODO: not sure what this code does
        if (newMockedResponse.request.query) {
            const queryWithoutConnection = utilities_1.removeConnectionDirectiveFromDocument(newMockedResponse.request.query);
            ts_invariant_1.invariant(queryWithoutConnection, "query is required");
            newMockedResponse.request.query = queryWithoutConnection;
            const query = utilities_1.removeClientSetsFromDocument(newMockedResponse.request.query);
            if (query) {
                newMockedResponse.request.query = query;
            }
        }
        return newMockedResponse;
    }
}
exports.OperationNameMockLink = OperationNameMockLink;
// Pass in multiple mocked responses, so that you can test flows that end up
// making multiple queries to the server.
// NOTE: The last arg can optionally be an `addTypename` arg.
function mockSingleLink(...mockedResponses) {
    // To pull off the potential typename. If this isn't a boolean, we'll just
    // set it true later.
    let maybeTypename = mockedResponses[mockedResponses.length - 1];
    let mocks = mockedResponses.slice(0, mockedResponses.length - 1);
    if (typeof maybeTypename !== "boolean") {
        mocks = mockedResponses;
        maybeTypename = true;
    }
    return new OperationNameMockLink(mocks, maybeTypename);
}
exports.mockSingleLink = mockSingleLink;
